#!/usr/bin/perl

# Description: Analyse network traffic (dump file).
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-13

# mapwocanalyse.pl - Analayse network traffic from dump file.
# Usage:
#   mapwocanalyse <dump_file> [working_directory]
# working_directory defaults to cwd.

# Result:
# Creates working_directory/Log and working_directory/Bin.

use strict;
use warnings;
use Net::Pcap;
use NetPacket::Ethernet;
use NetPacket::IP;
use NetPacket::TCP;
use Digest::MD5 qw(md5 md5_hex);
use IO::Uncompress::Gunzip qw(gunzip $GunzipError);
use Cwd;

# Global Variables
##################
my $BASE_DIR;
my $BIN_DIR;
my $TMP_DIR;
my $LOG_DIR;
my $SEG_DIR;
my $data = "";
my $data2;
my $infile;
my $outfile;
my $uncompress;
my $err;
my @filename;
my $logfile;
my $logfileh;
my $logfile404;
my $logfile302;
my $file = $ARGV[0];
my ($address, $netmask);
my $emtyflag;
my $binflag;
my $aus1;
my $header;
my $header1;
my $fname;
my $referer;
my $location;
my $host;
my $browser;
my $outfile1;
my $logdata;
my $ftime;
my $logheader;
my $oldheader=" ";

# Script Start.

# Set defaults.
$BASE_DIR=cwd;

if ( defined $ARGV[1] ) {
    $BASE_DIR=$ARGV[1];
}

$TMP_DIR=$BASE_DIR.'/TMP';
$LOG_DIR=$BASE_DIR.'/Log';

#"[SEGMENT]"){
#       $SEG_DIR=$argument[1];

$netmask=255.255.255.0;

# Create packet capture object on (file) device.

my @dirall = split(/\//,$file);
my @dir = split(/\./,$dirall[(scalar @dirall)-1]);

$logfile=$LOG_DIR."/Analyse.log";
$logfileh=$LOG_DIR."/header.log";
$logfile404=$LOG_DIR."/404.log";
$logfile302=$LOG_DIR."/302.log";

# Ensure that the Log directory does exist.
mkdir ("$LOG_DIR", 0777) unless -d "$LOG_DIR";

# Create log file with 'header' if it does not exist.
if (!(-e $logfileh)) {
    open(LOUT, ">> $logfileh") || die "Problem with $logfileh\n";
    print LOUT "0.0.0.0  0      0.0.0.0       Template\n";
    close(LOUT);
}
mkdir ("$BASE_DIR/Binary", 0777) unless -d "$BASE_DIR/Binary";
$dir[(scalar @dir)-2 ]=~ s/:/_/mg;
mkdir ($BASE_DIR."/".$dir[(scalar @dir)-2 ], 0777);
chdir ($BASE_DIR."/".$dir[(scalar @dir)-2 ]);
if ($file =~ /^\//) {
    $file=$file;
} else {
    $file=$BASE_DIR."/".$file;
}

# Write Log entry to Test.log.
my $object;
open(OUT,">> $LOG_DIR/Test.log");
print OUT "File:".$file."\n";
close(OUT);

{
# Globally set in/out record-separators.
$/ = undef;
$\ = "";
$| = 0;

$object = Net::Pcap::open_offline($file,\$err);
unless (defined $object) {
    die 'Unable to create packet capture on device ', $file, ' - ', $err;
}

# Call syn_packets on all packets.
Net::Pcap::loop($object, -1, \&syn_packets, '') ;
Net::Pcap::close($object);
}

$binflag=0;
# @filename gets populated in syn_packets like this:
# push @filename,$ip->{'src_ip'}."_".$tcp->{'src_port'}."_".$tcp->{'dest_port'});
foreach my $name (@filename) {
    #print "NAME:".$name."\n";
    my @dirall = split(/\//,$name);
    my @dir = split(/\./,$dirall[(scalar @dirall)-1]);
    mkdir ($dir[(scalar @dir)-1 ],0777);
    chdir ($dir[(scalar @dir)-1 ]);
    $emtyflag=0;
    print "Aufruf auswertung mit ../$name aus: ".$dir[(scalar @dir)-1 ]."\n";
    &auswertung ("../".$name);
    chdir ("..");
    #if ($emtyflag == 0) {rmdir ($BASE_DIR."/".$dir[(scalar @dir)-1 ]);}
}

if ($binflag != 0){
    open (IN,"< $file");
    open (OUT,"> $BASE_DIR/Binary/".$dirall[(scalar @dirall)-1]);
    while(<IN>){
    	print OUT $_;
    }
    close(IN);
    close(OUT);
}

# Run Antivirus software on extracts.
my $tmp =$BASE_DIR."/".$dir[(scalar @dir)-2 ];
#print "ANTIVIR-TMP: $tmp\n";
my $antivir = qx(/usr/bin/clamscan $tmp);
my @antivir_out=split(/\n/,$antivir);
open(OUT,">> $LOG_DIR/Antivir.log");
foreach my $av_line (@antivir_out){
    $av_line=~ s/^ ALERT:([\x00-\xff]{1,})<<</print OUT "$1\n";/eg;
}
close(OUT);

# Clean up, move logfile to logfile.old etc.
# In the following do not rm(dir) commands are commented out to debug.
if (! defined $ARGV[1]){
    foreach my $name (@filename) {
        print $name."\n";
        #system("rm $tmp/$name -R");
        print "Removed $tmp/$name .";
    }
    #system("rm $tmp/* -R");
    #system("cat $logfileh >> $logfileh.old");
    #system("rm $logfileh");
    #rmdir ($tmp);
}
else {
    system("cat $logfileh >> $logfileh.old");
}

exit 0;


# Sub programs
##############

=head2 METHODS

=over 1

=item syn_packets() USER_DATA HEADER PACKET

Callback for pcap::loop . Writes headers and traffic into files.

=cut

sub syn_packets {
    my ($user_data, $header, $packet) = @_;

    # Strip ethernet encapsulation of captured packet 

    my $ether_dec = NetPacket::Ethernet->decode($packet);
    my $ether_data = NetPacket::Ethernet::strip($packet);

    # Decode contents of TCP/IP packet contained within 
    # captured ethernet packet

    my $ip = NetPacket::IP->decode($ether_data);
    my $tcp = NetPacket::TCP->decode($ip->{'data'});

    my $peer_string=$ip->{'src_ip'}."_".$tcp->{'src_port'}."_".$tcp->{'dest_port'};

    # Print all out where its coming from and where its going to!
    my $tcpdata = $tcp->{'data'};
    #print "SRC:$ip->{'src_ip'}:$tcp->{'src_port'}  DEST:$ip->{'dest_ip'}:$tcp->{'dest_port'}    Length:".length ($tcp->{'data'})."  \n";

    # Open File according to string that should represent this communication (it doesnt, as dst-port is not unique).
	if ( (-e $peer_string) && (length ($tcp->{'data'}) >0) ) {
	    open (AUS ,">>".$peer_string);
	} else {
        my $peer_string_ext=$ip->{'dest_ip'}."_".$tcp->{'dest_port'}."_".$tcp->{'src_port'};
	    if ( (-e $peer_string_ext )  && (length ($tcp->{'data'}) >0) ) {
            # What does this mean? It is the second packet of this peer combo?
    		open (AUS ,">>".$peer_string_ext);
	    } else {
    		if (length ($tcp->{'data'}) >1) {
	    	    open (AUS ,">>".$peer_string);
	    	    push (@filename,$peer_string);
	    	}
            # ELSE? AUS is undefined?
	    }
	}
	if (length ($tcpdata) >1) {
	    if (($tcpdata=~ /^GET/) || ($tcpdata=~ /^POST/)) {
    		print AUS "SPLITMWOC";
	    	while (my ($i, $j) = each %$header){
		        if ($i eq "tv_sec") {
        			print AUS "TIME:".$j."\n";
        			my $h1=substr($tcpdata,0,500);
          			if(defined $h1){
           			    undef $fname; undef $referer; undef $host; undef $browser;
           			    $h1=~s/GET[\x00-\xff]{1,}\/(.[\x00-\xff]{1,}) HTTP\/...[\x0a\x0d]/$fname=$1;/eg;
           			    if(!(defined $fname)) {
                            $h1=~s/POST[\x00-\xff]{1,}\/(.[\x00-\xff]{1,}) HTTP\/...[\x0a\x0d]/$fname=$1;/eg;
                        }
           			    $h1=~s/Referer: http:\/\/(.[\x10-\xff]{1,})[\x0a\x0d]/$referer=$1;/eg;
          			    $h1=~s/Host: (.[\x10-\xff]{1,})[\x0a\x0d]/$host=$1;/eg;
           			    $h1=~s/User-Agent: (.[\x10-\xff]{1,})[\x0a\x0d]/$browser=substr($1,0,12);/eg;
       	    		}
        			if(!(defined $browser)){$browser="NODEF";} 
        			if(!(defined $fname)){$fname="";} 
        			if(!(defined $referer) && (defined $host)) {
            			open(NFH, ">>".$logfileh ) or die " Could not open $logfileh\n";
            			print NFH $ip->{'src_ip'}."	".$j."	$host/$fname	$browser\n";
            			close(NFH);
                        #print $ip->{'src_ip'}."	".$ip->{'dest_ip'}."TIME:".localtime($j)."	$host $fname	$browser  header: $h1\n";
    	    		}
    		    }
    		}
	    }
	    print AUS  "$tcpdata";
	    close(AUS);
	}
}


=item auswertung() INFILE

Reads in $data from INFILE, call strip afterwards.

=cut

sub auswertung {
    $infile = shift;
    $outfile = "_XXXX_";
    open(OFH, "<$infile") or die "Could not open $infile in auswertung.\n";
    binmode(OFH);
    seek (OFH, 0, 0);
    $data = <OFH>;
    close (OFH);
    #print "tcpflow file $infile is " . length($data) . " Bytes\n";
    &strip;
}


=item strip() 

Sets some variables, call other analysis functions afterwards.

=cut

sub strip{
	my @array = split(/SPLITMWOC/, $data);
	foreach my $aus (@array) {
	    undef ($aus1);
  	    my $aus2 = substr($aus,0,1500);
            my $tmp = $aus2;
        #$aus2 =~s/(^.[\x00-\xff]{1,}HTTP\/.[\x00-\xff]{1,}OK.[\x00-\xff]{1,})\x0d\x0a\x0d\x0a(.[\x00-\xff]{1,})/$header = $1; $aus1=$2;/eg;
   	    $aus2 =~s/(.[\x00-\xff]{1,}\x0d\x0aHTTP.[\x00-\xff]{1,}\x0d\x0aConnection.[\x00-\xff]{1,}\x0d\x0a\x0d\x0a)(.[\x00-\xff]{1,})/$header = $1; $aus1=$2;/eg;
	    if (!defined $aus1){
    		$aus2 =~s/(.[\x00-\xff]{1,}\x0d\x0aHTTP.[\x00-\xff]{1,}\x0d\x0aConten.[\x00-\xff]{1,}\x0d\x0a\x0d\x0a)(.[\x00-\xff]{1,})/$header = $1; $aus1=$2;/eg;
   	    }
	    if (!defined $aus1){
    		$aus2 =~s/(.[\x00-\xff]{1,}HTTP.[\x00-\xff]{1,}\x0d\x0aConten.[\x00-\xff]{1,}\x0d\x0a\x0d\x0a)(.[\x00-\xff]{1,})/$header = $1; $aus1=$2;/eg;
   	    }
   	    if ((length $aus > 1500) && (defined($aus1))) {
            $aus1=$aus1.substr($aus,1500);
        }
        # Data dezip.
	    if ((defined $aus1) && ($aus1 =~ /^\x1f\x8b\x08/)){
		    gunzip \$aus1 => \$uncompress;
		    $aus1 = $uncompress;
	    }
	    if (defined $aus1) {
#            print "AUS: $header\n";
    		my $oldh=$header;
    		&strip_exe;
    		$header=$oldh;
    		&strip_google_safebrowsing;
    		$header=$oldh;
    		&strip_google;
    		&strip_404;
                &strip_302($tmp);
	    } else {
               if($tmp =~ /HTTP\/1\.. 302/) {
                  &strip_302($tmp);
               }
            }

	}
}

=item make_outfile1() 

Sets $ftime, $fname, $referer, $host, $browser from headers found in $header.
Sets $logdata and $outfile1.

$logdata is file, time, host, referer, agent; tab-separated.

If no headers to process, set $outfile1="NO_HEADERINFO".

=cut

sub make_outfile1 {
	$header1=$header;
  	if(defined $header1){
    	undef($ftime);
    	undef($fname);
    	undef($referer);
        undef($location);
    	undef($host);
    	undef($browser);
		$header1=~s/TIME:(.[a-zA-Z0-9 :]{1,})/$ftime=$1;/eg;
		$header1=~s/GET[\x00-\xff]{1,}\/(.[\x00-\xff]{1,}) HTTP\/...[\x0a\x0d]/$fname=$1;/eg;
		$header1=~s/Referer: http:\/\/(.[\x10-\xff]{1,})[\x0a\x0d]/$referer=$1;/eg;
		$header1=~s/Host: (.[\x10-\xff]{1,})[\x0a\x0d]/$host=$1;/eg;
		$header1=~s/User-Agent: (.[\x10-\xff]{1,})[\x0a\x0d]/$browser=$1;/eg;
        # TODO Capture all redirects, not only http ones.
                $header1=~s/Location: (http:\/\/.[\x10-\xff]{1,})[\x0a\x0d]/$location=$1;/eg;
		if( !defined $ftime) { $ftime=0;}
		if( !defined $fname) { $fname="NO_NAME";}
		if( !defined $referer) { $referer="NO_REFERER";}
		if( !defined $host) { $host="NO_HOST";}
		if( !defined $browser) { $browser="NO_BROWSER";}
		$logdata=$infile."	".$ftime."	".$host."	".$fname."	".$referer."	".$browser;
                if(defined($location)) {
                  $logdata .= "      " . $location;
                }
        #$outfile1 = $host."_".$referer."_".$fname;
		if (length $host > 15) { $host = substr($host,0,13);}
		if (length $referer > 15) { $referer = substr($referer,0,13);}
		if (length $fname > 15) { $fname = substr($fname,0,13);}
		$outfile1 = $host."_".$referer."_".$fname;
		$outfile1=~s/[\/]/_/mg;
	}
	else {
	    $outfile1="NO_HEADERINFO";
	}
}


=item log_header() 

[Currently never called]

=cut

sub log_header {
    $header1=$header;
    if(defined $header1){
        $header1=~s/TIME:(.[a-zA-Z0-9 :]{1,})/$ftime=$1;/eg;
        if(defined $ftime){
            $header1=~s/GET[\x00-\xff]{1,}\/(.[\x00-\xff]{1,}) HTTP\/...[\x0a\x0d]/$fname=$1;/eg;
            $header1=~s/Referer: http:\/\/(.[\x10-\xff]{1,})[\x0a\x0d]/$referer=$1;/eg;
            $header1=~s/Host: (.[\x10-\xff]{1,})[\x0a\x0d]/$host=$1;/eg;
            $header1=~s/User-Agent: (.[\x10-\xff]{1,})[\x0a\x0d]/$browser=$1;/eg;
            if( !defined $fname) { $fname="NO_NAME";}
            if( !defined $referer) { $referer="NO_REFERER";}
            if( !defined $host) { $host="NO_HOST";}
            if( !defined $browser) { $host="NO_BROWSER";}
            $logheader=$infile."	".$ftime."	".$host."	".$fname."	".$referer."	".$browser;
            if( ($oldheader ne $infile) ) {
                open(NFH, ">>".$logfileh ) or die " Could not open $logfileh\n";
                print NFH $logheader."\n";
                close(NFH);
                $oldheader = $infile;
            }
        }
    }
	else {
        $logheader="NO_TIME";
	}
}


sub strip_exe {
    my $data1; 
    my $data3 = $aus1;
    #&log_header;
    undef $data1;
    $data3=~ s/(\x4D\x5A.[\x00-\xff]{1,250}\x68\x69\x73.[\x00-\xff]{1,250}\x50\x45\x00.[\x00-\xff]{1,})/$data1=$1;/eg;
    if(defined $data1) {
	    &make_outfile1;
        #my $md = md5_hex($aus1);
    	my $md = md5_hex($data1);
	    open(NFH, "> " . $outfile1."-".$md) or die "Could not open $outfile1-$md\n";
    	binmode(NFH);
    	$emtyflag++;
        #print NFH $aus1;
    	print NFH $data1;
    	close(NFH);
    	$data2 = `file -b $outfile1-$md`;
    	$data2 =~ s/[\r\n]+//g;
    	my $erg=&get_first_html;
    	open(NFH, ">>".$logfile) or die "could not open $logfile\n";
    	binmode(NFH);
    	if( !defined $erg) { $erg="NO_ERG";}
    	print NFH "[EXE]	$logdata	==>$data2<==	$md	$file	$erg\n";
    	close(NFH);
    	open(NFH, "> $BASE_DIR/Binary/".$outfile1."-".$md) or die "Could not open $outfile1";
    	binmode(NFH);
    	print NFH $data1;
    	close(NFH);
    	$binflag++;
    }
}

=item strip_google() 

Checks whether host in $header contains google-ana (case-insensitive).
If so, writes header and traffic to specific files.

=cut

sub strip_google {
    my $logheader;
    $header1=$header;
    #&log_header;
    if(defined $header1){
	    $header1=~s/TIME:(.[a-zA-Z0-9 :]{1,})/$ftime=$1;/eg;
    	if(defined $ftime){
	        $header1=~s/Referer: http:\/\/(.[\x10-\xff]{1,})[\x0a\x0d]/$referer=$1;/eg;
    	    $header1=~s/Host: (.[\x10-\xff]{1,})[\x0a\x0d]/$host=$1;/eg;
    	    if( !defined $referer) { $referer="NO_REFERER";}
    	    if( !defined $host) { $host="NO_HOST";}
    	    $logheader =$infile."	".$ftime."	Host:'".$host."'	Referer:'".$referer."'";
    	}
    	if( (defined $host) &&($host=~ /[Gg][Oo][Oo][Gg][Ll][Ee]-[Aa][Nn][Aa]/)) {
	        &make_outfile1;
        	my $md = md5_hex($aus1);
    	    open(NFH, "> "."Google-".$outfile1.$md)or die "Could not open Google-$outfile1$md\n";
    	    binmode(NFH);
    	    $emtyflag++;
    	    print NFH $aus1;
    	    close(NFH);
            #$data2 = `file -b $outfile1-$md`;
            #$data2 =~ s/[\r\n]+//g;
    	    my $erg=&get_first_html;
    	    open(NFH, ">>".$logfile) or die "Could not open $logfile\n" ;
	        binmode(NFH);
    	    print NFH "[GOOGLE]	$logheader\n";
    	    close(NFH);
    	    open(NFH, "> $BASE_DIR/Binary/GOOGLE".$outfile1."-".$md) or die "Could not open $outfile1";
    	    binmode(NFH);
            # ?!?!:
	        if(defined $header){
        		print NFH $header;
	        }
    	    print NFH $aus1;
    	    close(NFH);
    	    $binflag++;
    	}
    }
}


sub strip_javascript{
    if ($header =~ /[jJ]avascript/) {
	    &make_outfile1;
    	open(NFH, "> " . "$outfile1");
    	binmode(NFH);
    	$emtyflag++;
    	print NFH "$aus1";
    	close(NFH);
        open(NFH, ">>".$logfile) or die "Could not open $logfile\n" ;
    	binmode(NFH);
    	print NFH "[Javascript]	$logdata\n";
    	close(NFH);
    }
}


sub strip_google_safebrowsing{
    if (($header =~ /safebrowsing/) && ($header =~ /gethash/)){
        ##if ($header =~ /gethash/) {
        &make_outfile1;
        open(NFH, "> $BASE_DIR/Binary/"."$outfile1");
        binmode(NFH);
        $emtyflag++;
        print NFH "$header\n$aus1";
        close(NFH);
    	my $erg=&get_first_html;
        open(NFH, ">>".$logfile) or die "could not open $logfile\n" ;
        binmode(NFH);
        print NFH "[Google_Safebrowsing]	$logdata	$erg\n";
        close(NFH);
    }
}

=item strip_404() 

If $header contains the string ' 404' write to $logfile404.

=cut

sub strip_404{
    if (($header =~ / 404/)){
        &make_outfile1;
        #open(NFH, "> $BASE_DIR/Binary/"."$outfile1");
        #binmode(NFH);
        #$emtyflag++;
        #print NFH "$header\n$aus1";
        #close NFH;
    	my $erg=&get_first_html;
        open(NFH, ">>".$logfile404) or die "could not open $logfile\n" ;
        binmode(NFH);
        print NFH "[404]	$logdata	$erg\n";
        close(NFH);
    }
}

sub strip_302{

  $header = shift;
# print $header . "\n";

    if (($header =~ / 302/)){
        &make_outfile1;
        #open(NFH, "> $BASE_DIR/Binary/"."$outfile1");
        #binmode(NFH);
        #$emtyflag++;
        #print NFH "$header\n$aus1";
        #close NFH;
        my $erg=&get_first_html;
        open(NFH, ">>".$logfile302) or die "could not open $logfile\n" ;
        binmode(NFH);
        print NFH "[302]        $logdata	$erg\n";
        close(NFH);
    }
}
=item get_first_html() 

Might return something (the second field of the first line in logfile where the ip address is the
same address as in the current logdata record and...?).

=cut

sub get_first_html{
    my $erg;
    if (defined $logdata) {
        # Split log data at tabs.
        my @ldata = split(/\x09/,$logdata);
        my $ip_adr;
        $ldata[0]=~ s/^..\/([0-9.]{1,})_/$ip_adr=$1;/eg;
    	if( !defined $ldata[1]) { $ldata[1]=0;}
    	my $ti=$ldata[1];
    	my $brow=substr($ldata[5],0,12);
        # Set separator.
    	$\="\n";
    	open(INF, "<".$logfileh) or die " Could not open $logfileh\n";
    	my $endflag=0;
    	my $line;
    	while ($endflag==0 && !eof(INF)){
    	    $line=readline (INF);
    	    my @gdata = split(/\n/,$line);
    	    my $glength=@gdata-1 ;
    	    do{
        		my @hdata = split(/\x09/,$gdata[$glength]);
        		if ( ($ip_adr eq $hdata[0]) && ($endflag != 1) ){
        		    if ( (($ti-120) <= $hdata[1])  && ($ti >=$hdata[1]) ) {
            			if (( $brow eq $hdata[3]) &&( !($hdata[2] =~ /safebrowsing/))) {
        	    		    $erg=$hdata[2];
            			    $endflag=1;
            			}
        		    }
        		}
    		$glength--;
    	    } while (($glength ne 0));
         close(INF);
         # Unset separator.
    	 $\="";
	  }
	  return $erg;
    }
}
