# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Information about controlling a certain HoneyClient.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.


import ConfigParser

from hc import HC

class HCControl(HC):
    """Bundles HC meta- and control-information."""

    def __init__(self, filepath=None):
        """
        Initializes a HCControl.
        @param filepath: Reads control information from this file if given.
                         Otherwise, initializes values to empties.
        """

        HC.__init__(self, filepath)

        STR_HOMEPAGE = 'homepage'
        STR_CLOSE_WINDOW = 'close_window'
        STR_ENTER = 'enter'

        shortcut = {STR_HOMEPAGE : 'alt-home',
                    STR_CLOSE_WINDOW : 'alt-f4',
                    STR_ENTER : 'enter'}

        CONFIG_GROUP_CONTROL = 'control'
        CONFIG_GROUP_NETWORK = 'network'
        CONFIG_GROUP_KEYBOARD_SHORTCUTS = 'keyboard-shortcuts'

        if filepath is not None:
            config = ConfigParser.RawConfigParser(shortcut)
            config.read(filepath)
            self.image_file = config.get(CONFIG_GROUP_CONTROL,
                                         'image_file')
            self.image_snapshot = config.get(CONFIG_GROUP_CONTROL,
                                             'image_snapshot')
            self.image_memory = config.getint(CONFIG_GROUP_CONTROL,
                                           'image_memory')
            if config.has_option(CONFIG_GROUP_CONTROL, 'kvm_kernel'):
                self.kvmkernel = config.get(CONFIG_GROUP_CONTROL, 'kvm_kernel')
            else:
                self.kvmkernel = None
            self.hctype = config.get(CONFIG_GROUP_CONTROL, 'type')
            self.own_ip = config.get(CONFIG_GROUP_NETWORK, 'own_ip')
            self.dns_ip = config.get(CONFIG_GROUP_NETWORK, 'dns_ip')
            self.proxy_ip = config.get(CONFIG_GROUP_NETWORK, 'proxy_ip')
            self.gateway_ip = config.get(CONFIG_GROUP_NETWORK,
                                         'gateway_ip')
            self.homepage = config.get(CONFIG_GROUP_NETWORK, 'homepage')
            if not config.has_section(CONFIG_GROUP_KEYBOARD_SHORTCUTS):
                config.add_section(CONFIG_GROUP_KEYBOARD_SHORTCUTS)
            for key in shortcut.keys():
                shortcut[key] = config.get(CONFIG_GROUP_KEYBOARD_SHORTCUTS,
                                           key)
        else:
            self.image_file = None
            self.image_snapshot = None
            self.image_memory = None
            self.hctype = None
            self.own_ip = None
            self.dns_ip = None
            self.proxy_ip = None
            self.gateway_ip = None
            self.homepage = None

        # TODO Remove the need of shortcut definition in each and every
        #      HCControl-Instance, instead introduce proper defaulting
        #      and dynamically use the shortcuts dictionary, when available.
        self.SHORTCUT_HOMEPAGE = shortcut[STR_HOMEPAGE]
        self.SHORTCUT_CLOSE_WINDOW = shortcut[STR_CLOSE_WINDOW]
        self.SHORTCUT_ENTER = shortcut[STR_ENTER]

