# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Visits, Results and ResultType Classes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
# Andre Heinecke <aheinecke@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import datetime

# Map ResultTypes to severities

RESULT_TYPES={
        "unknown" : 999,
        "RegularTraffic" : 000,
        "404" : 100,
        "GoogleAnalytics" : 500,
        "MissingTraffic" : 600,
        "WindowsExecutable" : 500
        }

class ResultFile(object):
    def __init__(self):
        self.filepath = None
        self.sha1sum = None
        self.ssdeep = None

class ResultType(object):

    def __init__(self, name, severity):
        self.type_name = name
        self.severity = severity

class Result(object):

    def __init__(self, url=None, result_type=None, method='', date_time=None):
        """Creates a new result.
            url - The URL String that this Result relates to.
            result_type - The ResultType of this Result.
            method - The method string of this Result.
            date_time - Time of finding. Is set to datetime.now() if None.
        """
        self.url = url
        if date_time is None:
            self.date = datetime.datetime.now()
        else:
            self.date = date_time
        self.result_type = result_type
        self.text = None
        self.ip = None
        self.iframe = None
        self.method = method
        self.result_file = None

class Visit(object):
    def __init__(self):
        self.hcsid = ""
        self.results = []
        self.node = ""
        self.scan = None
        self.date = None
        self.outgoingIp = None
        self.filepath = None
        self.externalInfo = None
        self.urls = []
