# -*- coding: utf-8 -*-

# mapWOC
#
# Description: URL and URL Lists.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
from formencode import validators
from formencode.validators import CIDR
from datetime import datetime

class URL(object):

    def __init__(self, url=None):
        """URL object.  Assign a value to url."""
        self.url = url


class URLList(object):

    def __init__(self, name):
        self.name = name
        self.urls = []
        self.comment = ''
        self.date = datetime.now()
        self.deleted = False

    def delete(self):
        self.deleted = True

def create_URL_List(name, urls, comment = None):
    """Create a urllist from a linefeed seperated list of urls.
       Returns a pair (validurls, invalidurls) where the first item is
       an URLList Instance and the second is a list of Strings.
    """

    invalid_urls = []
    urllist = URLList(name)
    urllist.comment = comment or ''
    stripped_lines = [ line.strip() for line in urls.split() ]
    for line in stripped_lines:
        if not validate_URL(line):
            invalid_urls.append(line)
            continue
        urllist.urls.append(URL(line))
    return (urllist,invalid_urls)

def validate_URL(urlstring):
    """Checks if the given string urlstring is a valid URL."""

    if not urlstring or urlstring == "http://" or urlstring == "https://":
        # Invalidate valid but empty urls
        return False
    try:
        # Check if urlstring is valid
        u = validators.URL(add_http=False, check_exists=False)
        u.to_python(urlstring)
        return True
    except:
        # Check if urlstring is a valid ipv4 Address
        if urlstring.startswith("http://"):
            urlstring = urlstring.replace("http://","")
        elif urlstring.startswith("https://"):
            urlstring = urlstring.replace("https://","")
        else:
            return False
        cidr = CIDR()
        try:
            if len(urlstring.split(":")) > 1:
                port = urlstring.split(":", 1)[1]
                urlstring = urlstring.split(":", 1)[0]
                if port and int(port) > 65535:
                    return False
            cidr.to_python(urlstring.rstrip("/"))
            return True
        except:
            return False
        return False
