# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Process to poll a number of Nodes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import socket
from multiprocessing import Process, Queue
from time import sleep

from nodeconnection import NodeConnection


class NodePollProcess(Process):
    """Ask a number of nodes every poll_interval seconds about their state.
       The answers are put in the Queue (as dicts).
    """

    def __init__(self, queue, nodes, poll_interval=60):
        """Sets up (but doesnt start) the Process to periodically check for
           nodes state.


           nodes - A dict with nodename and NodeConnection instance.
           queue - A Queue that will fill up with answers (strings) of the
                   redirector.
           poll_interval - The interval in seconds to wait between two communication
                   attempts to the redirector.
        """

        Process.__init__(self)
        self.__poll_interval = poll_interval
        self.queue = queue
        self.nodes = nodes

    def run(self):
        """Ask the nodes about their state, put answer string in dict
           in queue, sleep given number of seconds, redo this.
           Forever.
        """

        while True:
            # TODO try/catch here instead of in ask_redirector
            # TODO proper error handling
            nodestates = {}
            for key,node in self.nodes.items():
                nodestates[key] = node.ask_decode('state')
                  
            self.queue.put(nodestates)
            sleep(self.__poll_interval)

