# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Process to poll a Redirector.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import socket
from multiprocessing import Process, Queue
from time import sleep
import json

from mapwoc.lib.mapwoc_json import mapwoc_from_json_hook as json_decode

# TODO use logging, eventually refactor with other "poll"-process
#      implementations.

def ask_redirector(question, host, port):
    """
    Asks a redirector and returns the answer as a string.

    @param question: The Question to ask.
    @param host: The host where the redirector resides.
    @param port: The port of the redirector (ctrl).
    @return Answer as string.
    """

    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((host,port))
        sock.send(question)
        received = sock.recv(8192)
        sock.close()
    except socket.error, msg:
        #logger.warning("Error when talking to redirector: %s" % msg)
        print "Redirector poll process got no answer. From %s:%d" % \
                (host,port)
        return ''
    return received


def ask_redirector_decode(question, host, port):
    """
    Asks a redirector, returns json-decoded answer.

    @param question: The Question to ask.
    @param host: The host where the redirector resides.
    @param port: The port of the redirector (ctrl).
    @return Decoded object.
    """

    return json.loads(ask_redirector(question, host, port),
                      object_hook=json_decode)

class RedirectorPollProcess(Process):
    """Ask a redirector every poll_interval seconds about its state.
       Its answer is put in the Queue.
    """

    def __init__(self, queue, poll_interval=60, host='localhost', port=8228):
        """Sets up (but doesnt start) the Process to periodically check for
           a redirectors state.

           queue - A Queue that will fill up with answers (strings) of the
                   redirector.
           poll_interval - The interval in seconds to wait between two communication
                   attempts to the redirector.
           host - The address at which the redirectorctlserver listens
           port - The port at which the redirectorctlserver listens
        """

        Process.__init__(self)
        self.__poll_interval = poll_interval
        self.queue = queue
        self.host = host
        self.port = port

    def run(self):
        """Ask redirector about its state, put answer string in queue, sleep
           given number of seconds, redo this.
           Forever.
        """

        while True:
            # TODO try/catch here instead of in ask_redirector
            # TODO proper error handling
            response = ask_redirector_decode('state_ext 10 10', self.host, self.port)
            self.queue.put(response)
            sleep(self.__poll_interval)

