import logging

from pylons import request, response, session, tmpl_context as c, url
from pylons.controllers.util import abort, redirect
from pylons.i18n.translation import _, ungettext

import formencode

from pylons.decorators import validate
from pylons.decorators.rest import restrict

import webhelpers.paginate as paginate

from mapwocgui.lib.base import BaseController, render
import mapwocgui.lib.helpers as h 

from mapwocgui.model import Session
from mapwoc.lib.url import URLList, URL, create_URL_List

log = logging.getLogger(__name__)

class NewURLListForm(formencode.Schema):
    """URLList Validation, dummy."""

    # Extra field is submit button, right now
    allow_extra_fields = True
    filter_extra_fields = True
    URLListName = formencode.validators.String(not_empty=True)
    URLListComment = formencode.validators.String()
    URLsText = formencode.validators.String(not_empty=True)

class UploadURLListForm(formencode.Schema):
    """URLList upload validation."""

    # Extra field is submit button, right now
    allow_extra_fields = True
    filter_extra_fields = True
    URLListUploadName = formencode.validators.String(not_empty=True)
    URLListUploadComment= formencode.validators.String()
    URLListFile = formencode.validators.String(not_empty=True)

class UrllistController(BaseController):

    def index(self):
        records = Session.query(URLList).filter_by(deleted=False)
        c.urllist_paginator = paginate.Page(records,
                                   page=int(request.params.get('page',1)),
                                   items_per_page=15,)
        return render('/urllist/index_urllist.mako')

    def new(self):
        return render('/urllist/index_urllist.mako')

    def handle_invalids(self, invalids):
        """ Sets a flash alert for invalid URLs """

        if not invalids:
            return
        if len(invalids) < 20:
            h.flash(_(u"URL list contained invalid URLs:"))
            for invalid in invalids:
                h.flash(invalid)
        else:
            h.flash(_(u"More then 20 URL's in the list were invalid."))
            h.flash(_(u"Please check the created list."))

    @restrict('POST')
    @validate(schema=UploadURLListForm(), form='new')
    def upload(self):
        """ Creates an URLList from a file. """

        filecontent = request.POST['URLListFile'].value
        urllist,invalids = create_URL_List(self.form_result['URLListUploadName'],
                                  filecontent,
                                  self.form_result['URLListUploadComment'])
        self.handle_invalids(invalids)
        Session.add(urllist)
        Session.commit()

        response.status_int = 302
        response.headers['location'] = h.url_for(controller='urllist',
                action='index', id=urllist.id)
        return "Moved temporarily"


    @restrict('POST')
    @validate(schema=NewURLListForm(),form='new')
    def create(self):
        """Creates an URLList from a request."""

        # Create from request
        urllist,invalids = create_URL_List(self.form_result['URLListName'],
                                  self.form_result['URLsText'],
                                  self.form_result['URLListComment'])
        self.handle_invalids(invalids)
        # Add to store
        Session.add(urllist)
        Session.commit()

        # Redirect
        # TODO use redirect_to from pylons.controller.utils ?
        response.status_int = 302
        response.headers['location'] = h.url_for(controller='urllist', action='index', id=urllist.id)
        return "Moved temporarily" 

    def view(self, id):
        query = Session.query(URLList)
        c.urllist = query.get(int(id))
        c.url_paginator = paginate.Page(c.urllist.urls,
                        page=int(request.params.get('page',1)),
                        items_per_page=100,)
        return render('/urllist/view_urllist.mako')

    #TODO check whether the url list is used in any scans.
    def delete(self, id):
        if id is None:
            abort(404)
        urllist_query = Session.query(URLList)
        urllist = urllist_query.filter_by(id=id).first()
        if urllist is None:
            abort(404)
        urllist.delete()
        Session.commit()
        response.status_int = 302
        response.headers['location'] = h.url_for(controller='urllist',action='index',id=urllist.id)
        return "Moved temporarily"        

