"""Setup the mapwocgui application"""
import logging

import pylons.test

from mapwocgui.config.environment import load_environment
from mapwocgui.model.meta import Session, Base, metadata
from mapwoc.lib.result import ResultType, RESULT_TYPES
import mapwocgui.model as model
from mapwoc.lib.model.metatable import meta_table
from hashlib import md5
from authkit.users.sqlalchemy_driver import UsersFromDatabase

log = logging.getLogger(__name__)

# Increment this number on every Database change
DB_VERSION = 1

def setup_app(command, conf, vars):
    """Database and pylons initialization."""

    # Don't reload the app if it was loaded under the testing environment
    if not pylons.test.pylonsapp:
        load_environment(conf.global_conf, conf.local_conf)

    # Add users to the model
    log.info("Adding the AuthKit model...")
    users = UsersFromDatabase(model)

    # Create the tables if they don't already exist
    log.info("Creating db")
    Base.metadata.create_all(bind=Session.bind)

    log.info("Database Version is %d " % DB_VERSION)
    stmt = meta_table.insert(bind=Session.bind)
    stmt.execute(version=DB_VERSION)

    log.info("Adding roles and users...")
    # Privileged users read/write access to scans/urls
    users.role_create("privileged")
    # Admin user read/write access to users
    users.role_create("admin")

    log.info("Adding ResultTypes...")
    for key in RESULT_TYPES:
        Session.add(ResultType(key, RESULT_TYPES[key]))

    log.info("Database set up")
    print("------------------------------------------------------------")
    print("| Please enter the credentials for the first admin account.")
    username = raw_input("Username: ")
    password = raw_input("Password: ")
    print("------------------------------------------------------------")
    users.user_create(username, password=md5(password).hexdigest())
    users.user_add_role(username, role="admin")
    users.user_add_role(username, role="privileged")
    Session.commit()

    print("Thank you, the admin account for %s is now set up." % username)
